package dev.cockscomb.client_android.views

import androidx.compose.foundation.layout.*
import androidx.compose.material.*
import androidx.compose.runtime.*
import app.src.main.java.dev.cockscomb.client_android.views.BlogListQuery
import com.apollographql.apollo.ApolloClient
import com.apollographql.apollo.coroutines.await
import com.apollographql.apollo.exception.ApolloException
import dev.cockscomb.client_android.LocalApolloClient

@OptIn(ExperimentalMaterialApi::class)
@Composable
fun BlogList() {
    val apolloClient = LocalApolloClient.current
    val blogs = remember {
        mutableStateOf<List<BlogListQuery.Node>>(value = emptyList())
    }

    LaunchedEffect(Unit) {
        val response = try {
            apolloClient.query(BlogListQuery(first = 10)).await()
        } catch (e: ApolloException) {
            return@LaunchedEffect
        }
        response.data?.blogs?.edges
            ?.mapNotNull { it?.node }
            ?.let { blogs.value = it }
    }
    
    Column {
        blogs.value.forEach { blog ->
            ListItem(text = { Text(blog.name) })
            Divider()
        }
    }
}